@@ -12,22 +12,23 @@ use super::{Token, Parser, BasicParseError};
 /// in which case the caller needs to check if the arguments’ parser is exhausted.
 /// Return `Ok((A, B))`, or `Err(())` for a syntax error.
 pub fn parse_nth<'i, 't>(input: &mut Parser<'i, 't>) -> Result<(i32, i32), BasicParseError<'i>> {
-    let token = input.next()?;
-    match token {
+    // FIXME: remove .clone() when lifetimes are non-lexical.
+    match input.next()?.clone() {
         Token::Number { int_value: Some(b), .. } => {
             Ok((0, b))
         }
-        Token::Dimension { int_value: Some(a), ref unit, .. } => {
+        Token::Dimension { int_value: Some(a), unit, .. } => {
             match_ignore_ascii_case! {
                 &unit,
                 "n" => Ok(try!(parse_b(input, a))),
                 "n-" => Ok(try!(parse_signless_b(input, a, -1))),
-                _ => {
-                    parse_n_dash_digits(&*unit).map(|val| (a, val))
+                _ => match parse_n_dash_digits(&*unit) {
+                    Ok(b) => Ok((a, b)),
+                    Err(()) => Err(BasicParseError::UnexpectedToken(Token::Ident(unit.clone())))
                 }
             }
         }
-        Token::Ident(ref value) => {
+        Token::Ident(value) => {
             match_ignore_ascii_case! { &value,
                 "even" => Ok((2, 0)),
                 "odd" => Ok((2, 1)),
@@ -35,48 +36,56 @@ pub fn parse_nth<'i, 't>(input: &mut Parser<'i, 't>) -> Result<(i32, i32), Basic
                 "-n" => Ok(try!(parse_b(input, -1))),
                 "n-" => Ok(try!(parse_signless_b(input, 1, -1))),
                 "-n-" => Ok(try!(parse_signless_b(input, -1, -1))),
-                _ => if value.starts_with("-") {
-                    parse_n_dash_digits(&value[1..]).map(|v| (-1, v))
-                } else {
-                    parse_n_dash_digits(&*value).map(|v| (1, v))
+                _ => {
+                    let (slice, a) = if value.starts_with("-") {
+                        (&value[1..], -1)
+                    } else {
+                        (&*value, 1)
+                    };
+                    match parse_n_dash_digits(slice) {
+                        Ok(b) => Ok((a, b)),
+                        Err(()) => Err(BasicParseError::UnexpectedToken(Token::Ident(value.clone())))
+                    }
                 }
             }
         }
-        Token::Delim('+') => match input.next_including_whitespace()? {
+        // FIXME: remove .clone() when lifetimes are non-lexical.
+        Token::Delim('+') => match input.next_including_whitespace()?.clone() {
             Token::Ident(value) => {
                 match_ignore_ascii_case! { &value,
-                    "n" => Ok(try!(parse_b(input, 1))),
-                    "n-" => Ok(try!(parse_signless_b(input, 1, -1))),
-                    _ => parse_n_dash_digits(&*value).map(|v| (1, v))
+                    "n" => parse_b(input, 1),
+                    "n-" => parse_signless_b(input, 1, -1),
+                    _ => match parse_n_dash_digits(&*value) {
+                        Ok(b) => Ok((1, b)),
+                        Err(()) => Err(BasicParseError::UnexpectedToken(Token::Ident(value.clone())))
+                    }
                 }
             }
-            t => return Err(BasicParseError::UnexpectedToken(t)),
+            token => Err(BasicParseError::UnexpectedToken(token)),
         },
-        _ => Err(()),
-    }.map_err(|()| BasicParseError::UnexpectedToken(token))
+        token => Err(BasicParseError::UnexpectedToken(token)),
+    }
 }
 
 
 fn parse_b<'i, 't>(input: &mut Parser<'i, 't>, a: i32) -> Result<(i32, i32), BasicParseError<'i>> {
-    let start_position = input.position();
-    let token = input.next();
-    match token {
-        Ok(Token::Delim('+')) => Ok(parse_signless_b(input, a, 1)?),
-        Ok(Token::Delim('-')) => Ok(parse_signless_b(input, a, -1)?),
-        Ok(Token::Number { has_sign: true, int_value: Some(b), .. }) => Ok((a, b)),
+    let start = input.state();
+    match input.next() {
+        Ok(&Token::Delim('+')) => parse_signless_b(input, a, 1),
+        Ok(&Token::Delim('-')) => parse_signless_b(input, a, -1),
+        Ok(&Token::Number { has_sign: true, int_value: Some(b), .. }) => Ok((a, b)),
         _ => {
-            input.reset(start_position);
+            input.reset(&start);
             Ok((a, 0))
         }
-    }.map_err(|()| BasicParseError::UnexpectedToken(token.unwrap()))
+    }
 }
 
 fn parse_signless_b<'i, 't>(input: &mut Parser<'i, 't>, a: i32, b_sign: i32) -> Result<(i32, i32), BasicParseError<'i>> {
-    let token = input.next()?;
-    match token {
+    match *input.next()? {
         Token::Number { has_sign: false, int_value: Some(b), .. } => Ok((a, b_sign * b)),
-        _ => Err(())
-    }.map_err(|()| BasicParseError::UnexpectedToken(token))
+        ref token => Err(BasicParseError::UnexpectedToken(token.clone()))
+    }
 }
 
 fn parse_n_dash_digits(string: &str) -> Result<i32, ()> {
