@@ -4,11 +4,10 @@
 
 // https://drafts.csswg.org/css-syntax/#parsing
 
-use compact_cow_str::CompactCowStr;
-use parser::{parse_until_before, parse_until_after, parse_nested_block};
+use cow_rc_str::CowRcStr;
+use parser::{parse_until_before, parse_until_after, parse_nested_block, ParserState};
 use std::ascii::AsciiExt;
-use std::ops::Range;
-use super::{Token, Parser, Delimiter, SourcePosition, ParseError, BasicParseError};
+use super::{Token, Parser, Delimiter, ParseError, BasicParseError, SourceLocation};
 
 
 /// Parse `!important`.
@@ -72,7 +71,7 @@ pub trait DeclarationParser<'i> {
     /// If `!important` can be used in a given context,
     /// `input.try(parse_important).is_ok()` should be used at the end
     /// of the implementation of this method and the result should be part of the return value.
-    fn parse_value<'t>(&mut self, name: CompactCowStr<'i>, input: &mut Parser<'i, 't>)
+    fn parse_value<'t>(&mut self, name: CowRcStr<'i>, input: &mut Parser<'i, 't>)
                        -> Result<Self::Declaration, ParseError<'i, Self::Error>>;
 }
 
@@ -112,7 +111,7 @@ pub trait AtRuleParser<'i> {
     /// The given `input` is a "delimited" parser
     /// that ends wherever the prelude should end.
     /// (Before the next semicolon, the next `{`, or the end of the current block.)
-    fn parse_prelude<'t>(&mut self, name: CompactCowStr<'i>, input: &mut Parser<'i, 't>)
+    fn parse_prelude<'t>(&mut self, name: CowRcStr<'i>, input: &mut Parser<'i, 't>)
                      -> Result<AtRuleType<Self::Prelude, Self::AtRule>, ParseError<'i, Self::Error>> {
         let _ = name;
         let _ = input;
@@ -238,10 +237,18 @@ where P: DeclarationParser<'i, Declaration = I, Error = E> +
 
     fn next(&mut self) -> Option<Result<I, PreciseParseError<'i, E>>> {
         loop {
-            let start_position = self.input.position();
-            match self.input.next_including_whitespace_and_comments() {
-                Ok(Token::WhiteSpace(_)) | Ok(Token::Comment(_)) | Ok(Token::Semicolon) => {}
-                Ok(Token::Ident(name)) => {
+            let start = self.input.state();
+            // FIXME: remove intermediate variable when lifetimes are non-lexical
+            let ident = match self.input.next_including_whitespace_and_comments() {
+                Ok(&Token::WhiteSpace(_)) | Ok(&Token::Comment(_)) | Ok(&Token::Semicolon) => continue,
+                Ok(&Token::Ident(ref name)) => Ok(Ok(name.clone())),
+                Ok(&Token::AtKeyword(ref name)) => Ok(Err(name.clone())),
+                Ok(token) => Err(token.clone()),
+                Err(_) => return None,
+            };
+            match ident {
+                Ok(Ok(name)) => {
+                    // Ident
                     return Some({
                         let parser = &mut self.parser;
                         // FIXME: https://github.com/rust-lang/rust/issues/42508
@@ -251,21 +258,23 @@ where P: DeclarationParser<'i, Declaration = I, Error = E> +
                         })
                     }.map_err(|e| PreciseParseError {
                         error: e,
-                        span: start_position..self.input.position()
+                        slice: self.input.slice_from(start.position()),
+                        location: start.source_location(),
                     }))
                 }
-                Ok(Token::AtKeyword(name)) => {
-                    return Some(parse_at_rule(start_position, name, self.input, &mut self.parser))
+                Ok(Err(name)) => {
+                    // At-keyword
+                    return Some(parse_at_rule(&start, name, self.input, &mut self.parser))
                 }
-                Ok(t) => {
+                Err(token) => {
                     return Some(self.input.parse_until_after(Delimiter::Semicolon,
-                                                             |_| Err(ParseError::Basic(BasicParseError::UnexpectedToken(t))))
+                                                             |_| Err(ParseError::Basic(BasicParseError::UnexpectedToken(token.clone()))))
                                 .map_err(|e| PreciseParseError {
                                     error: e,
-                                    span: start_position..self.input.position()
+                                    slice: self.input.slice_from(start.position()),
+                                    location: start.source_location(),
                                 }))
                 }
-                Err(_) => return None,
             }
         }
     }
@@ -333,30 +342,33 @@ where P: QualifiedRuleParser<'i, QualifiedRule = R, Error = E> +
 
     fn next(&mut self) -> Option<Result<R, PreciseParseError<'i, E>>> {
         loop {
-            let start_position = self.input.position();
-            match self.input.next_including_whitespace_and_comments() {
-                Ok(Token::WhiteSpace(_)) | Ok(Token::Comment(_)) => {}
-                Ok(Token::CDO) | Ok(Token::CDC) if self.is_stylesheet => {}
-                Ok(Token::AtKeyword(name)) => {
-                    let first_stylesheet_rule = self.is_stylesheet && !self.any_rule_so_far;
-                    self.any_rule_so_far = true;
-                    if first_stylesheet_rule && name.eq_ignore_ascii_case("charset") {
-                        let delimiters = Delimiter::Semicolon | Delimiter::CurlyBracketBlock;
-                        let _: Result<(), ParseError<()>> = self.input.parse_until_after(delimiters, |_| Ok(()));
-                    } else {
-                        return Some(parse_at_rule(start_position, name, self.input, &mut self.parser))
-                    }
-                }
-                Ok(_) => {
-                    self.any_rule_so_far = true;
-                    self.input.reset(start_position);
-                    return Some(parse_qualified_rule(self.input, &mut self.parser)
-                                .map_err(|e| PreciseParseError {
-                                    error: e,
-                                    span: start_position..self.input.position()
-                                }))
-                }
+            let start = self.input.state();
+            // FIXME: remove intermediate variable when lifetimes are non-lexical
+            let at_keyword = match self.input.next_including_whitespace_and_comments() {
+                Ok(&Token::WhiteSpace(_)) | Ok(&Token::Comment(_)) => continue,
+                Ok(&Token::CDO) | Ok(&Token::CDC) if self.is_stylesheet => continue,
+                Ok(&Token::AtKeyword(ref name)) => Some(name.clone()),
+                Ok(_) => None,
                 Err(_) => return None,
+            };
+            if let Some(name) = at_keyword {
+                let first_stylesheet_rule = self.is_stylesheet && !self.any_rule_so_far;
+                self.any_rule_so_far = true;
+                if first_stylesheet_rule && name.eq_ignore_ascii_case("charset") {
+                    let delimiters = Delimiter::Semicolon | Delimiter::CurlyBracketBlock;
+                    let _: Result<(), ParseError<()>> = self.input.parse_until_after(delimiters, |_| Ok(()));
+                } else {
+                    return Some(parse_at_rule(&start, name.clone(), self.input, &mut self.parser))
+                }
+            } else {
+                self.any_rule_so_far = true;
+                self.input.reset(&start);
+                return Some(parse_qualified_rule(self.input, &mut self.parser)
+                            .map_err(|e| PreciseParseError {
+                                error: e,
+                                slice: self.input.slice_from(start.position()),
+                                location: start.source_location(),
+                            }))
             }
         }
     }
@@ -369,13 +381,15 @@ pub fn parse_one_declaration<'i, 't, P, E>(input: &mut Parser<'i, 't>, parser: &
                                                      PreciseParseError<'i, E>>
                                            where P: DeclarationParser<'i, Error = E> {
     let start_position = input.position();
+    let start_location = input.current_source_location();
     input.parse_entirely(|input| {
-        let name = input.expect_ident()?;
+        let name = input.expect_ident()?.clone();
         input.expect_colon()?;
         parser.parse_value(name, input)
     }).map_err(|e| PreciseParseError {
         error: e,
-        span: start_position..input.position()
+        slice: input.slice_from(start_position),
+        location: start_location,
     })
 }
 
@@ -387,27 +401,36 @@ where P: QualifiedRuleParser<'i, QualifiedRule = R, Error = E> +
          AtRuleParser<'i, AtRule = R, Error = E> {
     input.parse_entirely(|input| {
         loop {
-            let start_position = input.position();
-            match input.next_including_whitespace_and_comments()? {
-                Token::WhiteSpace(_) | Token::Comment(_) => {}
-                Token::AtKeyword(name) => {
-                    return parse_at_rule(start_position, name, input, parser).map_err(|e| e.error)
-                }
-                _ => {
-                    input.reset(start_position);
-                    return parse_qualified_rule(input, parser)
-                }
+            let start = input.state();
+            // FIXME: remove intermediate variable when lifetimes are non-lexical
+            let at_keyword = match *input.next_including_whitespace_and_comments()? {
+                Token::WhiteSpace(_) | Token::Comment(_) => continue,
+                Token::AtKeyword(ref name) => Some(name.clone()),
+                _ => None
+            };
+            if let Some(name) = at_keyword {
+                return parse_at_rule(&start, name, input, parser).map_err(|e| e.error)
+            } else {
+                input.reset(&start);
+                return parse_qualified_rule(input, parser)
             }
         }
     })
 }
 
+/// A parse error with details of where it occured
 pub struct PreciseParseError<'i, E: 'i> {
+    /// Error details
     pub error: ParseError<'i, E>,
-    pub span: Range<SourcePosition>,
+
+    /// The relevant slice of the input.
+    pub slice: &'i str,
+
+    /// The line number and column number of the start of the relevant input slice.
+    pub location: SourceLocation,
 }
 
-fn parse_at_rule<'i: 't, 't, P, E>(start_position: SourcePosition, name: CompactCowStr<'i>,
+fn parse_at_rule<'i: 't, 't, P, E>(start: &ParserState, name: CowRcStr<'i>,
                                    input: &mut Parser<'i, 't>, parser: &mut P)
                                    -> Result<<P as AtRuleParser<'i>>::AtRule, PreciseParseError<'i, E>>
                                    where P: AtRuleParser<'i, Error = E> {
@@ -419,44 +442,49 @@ fn parse_at_rule<'i: 't, 't, P, E>(start_position: SourcePosition, name: Compact
     match result {
         Ok(AtRuleType::WithoutBlock(rule)) => {
             match input.next() {
-                Ok(Token::Semicolon) | Err(_) => Ok(rule),
-                Ok(Token::CurlyBracketBlock) => Err(PreciseParseError {
+                Ok(&Token::Semicolon) | Err(_) => Ok(rule),
+                Ok(&Token::CurlyBracketBlock) => Err(PreciseParseError {
                     error: ParseError::Basic(BasicParseError::UnexpectedToken(Token::CurlyBracketBlock)),
-                    span: start_position..input.position(),
+                    slice: input.slice_from(start.position()),
+                    location: start.source_location(),
                 }),
                 Ok(_) => unreachable!()
             }
         }
         Ok(AtRuleType::WithBlock(prelude)) => {
             match input.next() {
-                Ok(Token::CurlyBracketBlock) => {
+                Ok(&Token::CurlyBracketBlock) => {
                     // FIXME: https://github.com/rust-lang/rust/issues/42508
                     parse_nested_block::<'i, 't, _, _, _>(input, move |input| parser.parse_block(prelude, input))
                         .map_err(|e| PreciseParseError {
                             error: e,
-                            span: start_position..input.position(),
+                            slice: input.slice_from(start.position()),
+                            location: start.source_location(),
                         })
                 }
-                Ok(Token::Semicolon) => Err(PreciseParseError {
+                Ok(&Token::Semicolon) => Err(PreciseParseError {
                     error: ParseError::Basic(BasicParseError::UnexpectedToken(Token::Semicolon)),
-                    span: start_position..input.position()
+                    slice: input.slice_from(start.position()),
+                    location: start.source_location(),
                 }),
                 Err(e) => Err(PreciseParseError {
                     error: ParseError::Basic(e),
-                    span: start_position..input.position(),
+                    slice: input.slice_from(start.position()),
+                    location: start.source_location(),
                 }),
                 Ok(_) => unreachable!()
             }
         }
         Ok(AtRuleType::OptionalBlock(prelude)) => {
             match input.next() {
-                Ok(Token::Semicolon) | Err(_) => Ok(parser.rule_without_block(prelude)),
-                Ok(Token::CurlyBracketBlock) => {
+                Ok(&Token::Semicolon) | Err(_) => Ok(parser.rule_without_block(prelude)),
+                Ok(&Token::CurlyBracketBlock) => {
                     // FIXME: https://github.com/rust-lang/rust/issues/42508
                     parse_nested_block::<'i, 't, _, _, _>(input, move |input| parser.parse_block(prelude, input))
                         .map_err(|e| PreciseParseError {
                             error: e,
-                            span: start_position..input.position(),
+                            slice: input.slice_from(start.position()),
+                            location: start.source_location(),
                         })
                 }
                 _ => unreachable!()
@@ -465,12 +493,13 @@ fn parse_at_rule<'i: 't, 't, P, E>(start_position: SourcePosition, name: Compact
         Err(error) => {
             let end_position = input.position();
             match input.next() {
-                Ok(Token::CurlyBracketBlock) | Ok(Token::Semicolon) | Err(_) => {},
+                Ok(&Token::CurlyBracketBlock) | Ok(&Token::Semicolon) | Err(_) => {},
                 _ => unreachable!()
             };
             Err(PreciseParseError {
                 error: error,
-                span: start_position..end_position,
+                slice: input.slice(start.position()..end_position),
+                location: start.source_location(),
             })
         }
     }
@@ -484,7 +513,7 @@ fn parse_qualified_rule<'i, 't, P, E>(input: &mut Parser<'i, 't>, parser: &mut P
     let prelude = parse_until_before::<'i, 't, _, _, _>(input, Delimiter::CurlyBracketBlock, |input| {
         parser.parse_prelude(input)
     });
-    match input.next()? {
+    match *input.next()? {
         Token::CurlyBracketBlock => {
             // Do this here so that we consume the `{` even if the prelude is `Err`.
             let prelude = prelude?;
